//go:build !runOnTask

// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0
package jobqueue

import (
	"gorm.io/gorm"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
)

//
//func startTestMySQLDockerImageAndContainer(t *testing.T, port string, ctx context.Context) error {
//	t.Helper()
//
//	cli, err := client.NewClientWithOpts(client.FromEnv, client.WithAPIVersionNegotiation())
//	if err != nil {
//		return err
//	}
//
//	imageName := "mysql:8"
//
//	reader, err := cli.ImagePull(ctx, imageName, types.ImagePullOptions{})
//	if err != nil {
//		return err
//	}
//
//	// if debug image pull, comment out the following lines
//	//_, _ = io.Copy(os.Stdout, reader)
//	_ = reader
//
//	hostConfig := &container.HostConfig{
//		PortBindings: nat.PortMap{
//			"3306/tcp": []nat.PortBinding{
//				{
//					HostIP:   DOCKER_TEST_HOST_IP,
//					HostPort: port,
//				},
//			},
//		},
//	}
//
//	resp, err := cli.ContainerCreate(ctx, &container.Config{
//		Image: imageName,
//		Env: []string{
//			"MYSQL_ROOT_PASSWORD=secret",
//			"MYSQL_USER=user",
//			"MYSQL_PASSWORD=secret",
//			"MYSQL_DATABASE=test",
//		},
//	}, hostConfig, nil, nil, "")
//
//	if err != nil {
//		return err
//	}
//
//	if err := cli.ContainerStart(ctx, resp.ID, types.ContainerStartOptions{}); err != nil {
//		return err
//	}
//
//	go func() {
//		<-ctx.Done()
//
//		timeout := 0
//		stopOptions := container.StopOptions{
//			Timeout: &timeout,
//			Signal:  "SIGKILL",
//		}
//		newCtx, _ := context.WithTimeout(context.Background(), 60*time.Second)
//		if err := cli.ContainerStop(newCtx, resp.ID, stopOptions); err != nil {
//			t.Errorf("ContainerStop returned error: %v", err)
//		}
//		if err := cli.ContainerRemove(newCtx, resp.ID, types.ContainerRemoveOptions{
//			Force: true,
//		}); err != nil {
//			t.Errorf("ContainerRemove returned error: %v", err)
//		}
//
//	}()
//
//	statusCh, errCh := cli.ContainerWait(ctx, resp.ID, container.WaitConditionNotRunning)
//	select {
//	case err := <-errCh:
//		if err != nil {
//			// empty error means container exited normally (see container_wait.go)
//			if err.Error() == "" {
//				return nil
//			}
//
//			return err
//		}
//	case <-statusCh:
//
//	}
//
//	return nil
//}

func TestSaveJobWithSQLite(t *testing.T) {

	gormDB, err := gorm.Open(sqlite.Open("file::memory:?cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("a error occurred while opening the database: %v", err)
	}

	manager := &Manager{database: gormDB}
	saver := NewDBSaver().SetManager(manager)

	// Starte den DBSaver
	p := StartDBSaver(saver)

	ready := make(chan struct{})

	Then[bool, bool](p, func(value bool) (bool, error) {
		close(ready)
		return value, nil
	}, func(e error) error {
		close(ready)
		Error("Error while starting db saver", "error", err)
		return nil
	})

	<-ready

	jobID := JobID("testJob")
	job := NewJob[CounterResult](jobID, &CounterRunnable{})

	err = saver.SaveJob(job)
	assert.NoError(t, err)

	time.Sleep(100 * time.Millisecond)

	saver.Stop()

	var count int64
	gormDB.Model(&JobPersistence{}).Count(&count)
	assert.Equal(t, int64(1), count, "It should be 1 job in the database")

	// get job from database
	var jobFromDB JobPersistence
	gormDB.First(&jobFromDB, "id = ?", jobID)
	assert.Equal(t, jobID, jobFromDB.ID, "JobID should be the same")

}
