// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package jobqueue

// InstantScheduler is a scheduler that schedules a job instantly
type InstantScheduler struct{}

func (s *InstantScheduler) Schedule(job GenericJob, eventBus *EventBus) error {
	if !job.IsPaused() {
		eventBus.Publish(QueueJob, job)
	}
	return nil
}

func (s *InstantScheduler) GetType() string {
	return "Instant"
}

func (s *InstantScheduler) IsAdHoc() bool {
	return true
}

func (s *InstantScheduler) Cancel(id JobID) error {
	return nil
}

func (s *InstantScheduler) CancelAll() error {
	return nil
}

func (s *InstantScheduler) JobExists(id JobID) bool {
	return false
}

func (s *InstantScheduler) GetPersistence() SchedulerPersistence {
	return SchedulerPersistence{
		Type: s.GetType(),
	}
}
